# -*- coding: utf-8 -*-

from __future__ import print_function
import psycopg2
import psycopg2.extras
import difflib


class Checker:
    """This class is used to compare 2 Postgres databases and show the
    differences."""

    def __init__(
            self, pg_service1, pg_service2,
            exclude_schema=None, ignore_list=None, verbose_level=1):
        """Constructor

        Parameters
        ----------
        pg_service1: str
            The name of the postgres service (defined in pg_service.conf)
            related to the first db to be compared
        pg_service2: str
            The name of the postgres service (defined in pg_service.conf)
            related to the first db to be compared
        ignore_list: list(str)
            List of elements to be ignored in check (ex. tables, columns,
            views, ...)
        exclude_schema: list of strings
            List of schemas to be ignored in check.
        verbose_level: int
            verbose level, 0 -> nothing, 1 -> print first 80 char of each
            difference, 2 -> print all the difference details
        """

        self.conn1 = psycopg2.connect("service={0}".format(pg_service1))
        self.cur1 = self.conn1.cursor()

        self.conn2 = psycopg2.connect("service={0}".format(pg_service2))
        self.cur2 = self.conn2.cursor()

        self.ignore_list = ignore_list
        self.exclude_schema = "('information_schema'"
        if exclude_schema is not None:
            for schema in exclude_schema:
                self.exclude_schema += ", '{}'".format(schema)
        self.exclude_schema += ")"

        self.verbose_level = verbose_level

    def run_checks(self):
        """Run all the checks functions.

            Returns
            -------
            bool
                True if all the checks are true
                False otherwise
            dict
                Dictionary of lists of differences
        """

        result = True
        differences_dict = {}

        if 'tables' not in self.ignore_list:
            tmp_result, differences_dict['tables'] = self.check_tables()
            result = False if not tmp_result else result
        if 'columns' not in self.ignore_list:
            tmp_result, differences_dict['columns'] = self.check_columns(
                'views' not in self.ignore_list)
            result = False if not tmp_result else result
        if 'constraints' not in self.ignore_list:
            tmp_result, differences_dict['constraints'] = \
                self.check_constraints()
            result = False if not tmp_result else result
        if 'views' not in self.ignore_list:
            tmp_result, differences_dict['views'] = self.check_views()
            result = False if not tmp_result else result
        if 'sequences' not in self.ignore_list:
            tmp_result, differences_dict['sequences'] = self.check_sequences()
            result = False if not tmp_result else result
        if 'indexes' not in self.ignore_list:
            tmp_result, differences_dict['indexes'] = self.check_indexes()
            result = False if not tmp_result else result
        if 'triggers' not in self.ignore_list:
            tmp_result, differences_dict['triggers'] = self.check_triggers()
            result = False if not tmp_result else result
        if 'functions' not in self.ignore_list:
            tmp_result, differences_dict['functions'] = self.check_functions()
            result = False if not tmp_result else result
        if 'rules' not in self.ignore_list:
            tmp_result, differences_dict['rules'] = self.check_rules()
            result = False if not tmp_result else result
        if self.verbose_level == 0:
            differences_dict = None
        return result, differences_dict

    def check_tables(self):
        """Check if the tables are equals.

            Returns
            -------
            bool
                True if the tables are the same
                False otherwise
            list
                A list with the differences
        """
        query = """SELECT table_schema, table_name
                FROM information_schema.tables
                WHERE table_schema NOT IN {}
                    AND table_schema NOT LIKE 'pg\_%'
                    AND table_type NOT LIKE 'VIEW'
                ORDER BY table_schema, table_name
                """.format(self.exclude_schema)

        return self.__check_equals(query)

    def check_columns(self, check_views=True):
        """Check if the columns in all tables are equals.

            Parameters
            ----------
            check_views: bool
                if True, check the columns of all the tables and views, if
                False check only the columns of the tables

            Returns
            -------
            bool
                True if the columns are the same
                False otherwise
            list
                A list with the differences
        """
        if check_views:
            query = """WITH table_list AS (
                SELECT table_schema, table_name
                FROM information_schema.tables
                WHERE table_schema NOT IN {}
                    AND table_schema NOT LIKE 'pg\_%'
                ORDER BY table_schema,table_name
                )
                SELECT isc.table_schema, isc.table_name, column_name,
                    column_default, is_nullable, data_type,
                    character_maximum_length::text, numeric_precision::text,
                    numeric_precision_radix::text, datetime_precision::text
                FROM information_schema.columns isc,
                table_list tl
                WHERE isc.table_schema = tl.table_schema
                    AND isc.table_name = tl.table_name
                ORDER BY isc.table_schema, isc.table_name, column_name
                """.format(self.exclude_schema)

        else:
            query = """WITH table_list AS (
                SELECT table_schema, table_name
                FROM information_schema.tables
                WHERE table_schema NOT IN {}
                    AND table_schema NOT LIKE 'pg\_%'
                    AND table_type NOT LIKE 'VIEW'
                ORDER BY table_schema,table_name
                )
                SELECT isc.table_schema, isc.table_name, column_name,
                    column_default, is_nullable, data_type,
                    character_maximum_length::text, numeric_precision::text,
                    numeric_precision_radix::text, datetime_precision::text
                FROM information_schema.columns isc,
                table_list tl
                WHERE isc.table_schema = tl.table_schema
                    AND isc.table_name = tl.table_name
                ORDER BY isc.table_schema, isc.table_name, column_name
                """.format(self.exclude_schema)

        return self.__check_equals(query)

    def check_constraints(self):
        """Check if the constraints are equals.

            Returns
            -------
            bool
                True if the constraints are the same
                False otherwise
            list
                A list with the differences
        """
        query = """ select
                        tc.constraint_name,
                        tc.constraint_schema || '.' || tc.table_name || '.' ||
                            kcu.column_name as physical_full_name,
                        tc.constraint_schema,
                        tc.table_name,
                        kcu.column_name,
                        ccu.table_name as foreign_table_name,
                        ccu.column_name as foreign_column_name,
                        tc.constraint_type
                    from information_schema.table_constraints as tc
                    join information_schema.key_column_usage as kcu on
                        (tc.constraint_name = kcu.constraint_name and
                        tc.table_name = kcu.table_name)
                    join information_schema.constraint_column_usage as ccu on
                        ccu.constraint_name = tc.constraint_name
                    ORDER BY tc.constraint_schema, physical_full_name,
                        tc.constraint_name, foreign_table_name,
                        foreign_column_name  """

        return self.__check_equals(query)

    def check_views(self):
        """Check if the views are equals.

            Returns
            -------
            bool
                True if the views are the same
                False otherwise
            list
                A list with the differences
        """
        query = """
        SELECT table_name, REPLACE(view_definition,'"','')
        FROM INFORMATION_SCHEMA.views
        WHERE table_schema NOT IN {}
        AND table_schema NOT LIKE 'pg\_%'
        AND table_name not like 'vw_export_%'
        ORDER BY table_schema, table_name
        """.format(self.exclude_schema)

        return self.__check_equals(query)

    def check_sequences(self):
        """Check if the sequences are equals.

            Returns
            -------
            bool
                True if the sequences are the same
                False otherwise
            list
                A list with the differences
        """
        query = """
        SELECT c.relname
        FROM pg_class c
        WHERE c.relkind = 'S'
        ORDER BY c.relname"""

        return self.__check_equals(query)

    def check_indexes(self):
        """Check if the indexes are equals.

            Returns
            -------
            bool
                True if the indexes are the same
                False otherwise
            list
                A list with the differences
        """

        query = """
        select
            t.relname as table_name,
            i.relname as index_name,
            a.attname as column_name
        from
            pg_class t,
            pg_class i,
            pg_index ix,
            pg_attribute a
        where
            t.oid = ix.indrelid
            and i.oid = ix.indexrelid
            and a.attrelid = t.oid
            and a.attnum = ANY(ix.indkey)
            and t.relkind = 'r'
            AND t.relname NOT IN ('information_schema')
            AND t.relname NOT LIKE 'pg\_%'
        order by
            t.relname,
            i.relname,
            a.attname
        """
        return self.__check_equals(query)

    def check_triggers(self):
        """Check if the triggers are equals.

            Returns
            -------
            bool
                True if the triggers are the same
                False otherwise
            list
                A list with the differences
        """
        query = """
        WITH trigger_list AS (
            select tgname from pg_trigger
            GROUP BY tgname
        )
        select pp.prosrc, p.relname
        from pg_trigger t, pg_proc pp, trigger_list tl, pg_class p
        where pp.oid = t.tgfoid
            and t.tgname = tl.tgname
            AND t.tgrelid = p.oid
            and  SUBSTR(p.relname, 1, 3) != 'vw_'
            -- We cannot check for vw_ views,
            -- because they are created after that script
        ORDER BY p.relname, /*t.tgname, */pp.prosrc"""

        return self.__check_equals(query)

    def check_functions(self):
        """Check if the functions are equals.

            Returns
            -------
            bool
                True if the functions are the same
                False otherwise
            list
                A list with the differences
        """
        query = """
        SELECT routines.routine_name, parameters.data_type,
            routines.routine_definition
        FROM information_schema.routines
        JOIN information_schema.parameters
        ON routines.specific_name=parameters.specific_name
        WHERE routines.specific_schema NOT IN {}
            AND routines.specific_schema NOT LIKE 'pg\_%'
        ORDER BY routines.routine_name, parameters.data_type,
            routines.routine_definition, parameters.ordinal_position
            """.format(self.exclude_schema)

        return self.__check_equals(query)

    def check_rules(self):
        """Check if the rules are equals.

            Returns
            -------
            bool
                True if the rules are the same
                False otherwise
            list
                A list with the differences
        """
        query = """
        select n.nspname as rule_schema,
        c.relname as rule_table,
        case r.ev_type
            when '1' then 'SELECT'
            when '2' then 'UPDATE'
            when '3' then 'INSERT'
            when '4' then 'DELETE'
            else 'UNKNOWN'
        end as rule_event
        from pg_rewrite r
        join pg_class c on r.ev_class = c.oid
        left join pg_namespace n on n.oid = c.relnamespace
        left join pg_description d on r.oid = d.objoid
        WHERE n.nspname NOT IN {}
            AND n.nspname NOT LIKE 'pg\_%'
        ORDER BY n.nspname, c.relname, rule_event
        """.format(self.exclude_schema)

        return self.__check_equals(query)

    def __check_equals(self, query):
        """Check if the query results on the two databases are equals.

            Returns
            -------
            bool
                True if the results are the same
                False otherwise
            list
                A list with the differences
        """
        self.cur1.execute(query)
        records1 = self.cur1.fetchall()

        self.cur2.execute(query)
        records2 = self.cur2.fetchall()

        result = True
        differences = []

        d = difflib.Differ()
        records1 = [str(x) for x in records1]
        records2 = [str(x) for x in records2]

        for line in d.compare(records1, records2):
            if line[0] in ('-', '+'):
                result = False
                if self.verbose_level == 1:
                    differences.append(line[0:79])
                elif self.verbose_level == 2:
                    differences.append(line)

        return result, differences



# -*- coding: utf-8 -*-

from abc import ABCMeta, abstractmethod


class DeltaPy(metaclass=ABCMeta):
    """This abstract class must be instantiated by the delta.py classes"""

    def __init__(
            self, current_db_version, delta_dir, delta_dirs, pg_service, upgrades_table):
        """Constructor, receive some useful parameters accesible from the
        subclasses als propperties.

        Parameters
        ----------
        current_db_version: str
            The current db version
        pg_service: str
            The name of the postgres service (defined in pg_service.conf)
            related to the db
        upgrades_table: str
            The name of the table (int the format schema.name) where the
            informations about the upgrades are stored
        delta_dir: str
            The path to the directory where this delta file is stored
        delta_dirs: list(str)
            The paths to directories where delta files are stored
        """

        self.__current_db_version = current_db_version
        self.__delta_dir = delta_dir
        self.__delta_dirs = delta_dirs
        self.__pg_service = pg_service
        self.__upgrades_table = upgrades_table

    @abstractmethod
    def run(self):
        """This method must be implemented in the subclasses. It is called
        when the delta.py file is runned by Upgrader class"""
        pass

    @property
    def current_db_version(self):
        """Return the current db version"""
        return self.__current_db_version

    @property
    def delta_dir(self):
        """Return the path of the delta directory including this delta"""
        return self.__delta_dir

    @property
    def delta_dirs(self):
        """Return the paths of the delta directories"""
        return self.__delta_dirs

    @property
    def pg_service(self):
        """Return the name of the postgres service"""
        return self.__pg_service

    @property
    def upgrades_table(self):
        """Return the name of the upgrades information table"""
        return self.__upgrades_table

    def write_message(self, text):
        """Print a message from the subclass.

        Parameters
        ----------
        text: str
            The message to print
        """
        print('Message from {}: {}'.format(self.__class__.__name__, text))



import unittest

import psycopg2
import psycopg2.extras
from pum.core.checker import Checker

pg_service1 = 'pum_test_1'
pg_service2 = 'pum_test_2'

class TestChecker(unittest.TestCase):
    """Test the class Checker.

    2 pg_services related to 2 empty db, needed for test:
        pum_test_1
        pum_test_2
    """


    def tearDown(self):
        del self.checker

        self.cur1.execute('DROP SCHEMA IF EXISTS schema_foo CASCADE;')
        self.conn1.commit()
        self.conn1.close()

        self.cur2.execute('DROP SCHEMA IF EXISTS schema_foo CASCADE;')
        self.conn2.commit()
        self.conn2.close()

    def setUp(self):

        self.conn1 = psycopg2.connect("service={0}".format(pg_service1))
        self.cur1 = self.conn1.cursor()

        self.cur1.execute('DROP SCHEMA IF EXISTS schema_foo CASCADE;'
                          'CREATE SCHEMA schema_foo;')
        self.conn1.commit()

        self.conn2 = psycopg2.connect("service={0}".format(pg_service2))
        self.cur2 = self.conn2.cursor()

        self.checker = Checker(pg_service1, pg_service2)

        self.cur2.execute('DROP SCHEMA IF EXISTS schema_foo CASCADE;'
                          'CREATE SCHEMA schema_foo;')
        self.conn2.commit()

    def test_check_tables(self):
        self.cur1.execute('CREATE TABLE schema_foo.bar (id integer);')
        self.conn1.commit()

        result, differences = self.checker.check_tables()
        self.assertFalse(result)

        self.cur2.execute('CREATE TABLE schema_foo.bar (id integer);')
        self.conn2.commit()

        result, differences = self.checker.check_tables()
        self.assertTrue(result)

    def test_check_columns(self):
        self.cur1.execute(
            'CREATE TABLE schema_foo.bar '
            '(id smallint, value integer, name varchar(100));')
        self.conn1.commit()

        result, differences = self.checker.check_columns()
        self.assertFalse(result)

        self.cur2.execute(
            'CREATE TABLE schema_foo.bar '
            '(id integer, value integer, name varchar(100));')
        self.conn2.commit()

        result, differences = self.checker.check_columns()
        self.assertFalse(result)

        self.cur2.execute(
            'ALTER TABLE schema_foo.bar '
            'ALTER COLUMN id SET DATA TYPE smallint;')
        self.conn2.commit()

        result, differences = self.checker.check_columns()
        self.assertTrue(result)

    def test_check_constraints(self):

        self.cur1.execute(
            'CREATE TABLE schema_foo.bar '
            '(id smallint, value integer, name varchar(100), PRIMARY KEY(id));')
        self.conn1.commit()

        result, differences = self.checker.check_constraints()
        self.assertFalse(result)

        self.cur2.execute(
            'CREATE TABLE schema_foo.bar '
            '(id smallint, value integer, name varchar(100));')
        self.conn2.commit()

        result, differences = self.checker.check_constraints()
        self.assertFalse(result)

        self.cur2.execute(
            'ALTER TABLE schema_foo.bar DROP COLUMN id;'
            'ALTER TABLE schema_foo.bar ADD COLUMN id smallint PRIMARY KEY')
        self.conn2.commit()

        result, differences = self.checker.check_constraints()
        self.assertTrue(result)

    def test_check_views(self):

        self.cur1.execute("CREATE VIEW schema_foo.bar AS SELECT 'foobar';")
        self.conn1.commit()

        result, differences = self.checker.check_views()
        self.assertFalse(result)

        self.cur2.execute("CREATE VIEW schema_foo.bar AS SELECT 'foobar';")
        self.conn2.commit()

        result, differences = self.checker.check_views()
        self.assertTrue(result)

    def test_check_sequences(self):
        self.cur1.execute('DROP SEQUENCE IF EXISTS serial CASCADE;')
        self.conn1.commit()
        self.cur2.execute('DROP SEQUENCE IF EXISTS serial CASCADE;')
        self.conn2.commit()

        result, differences = self.checker.check_sequences()
        self.assertTrue(result)

        self.cur1.execute('CREATE SEQUENCE serial START 101;')
        self.conn1.commit()

        result, differences = self.checker.check_sequences()
        self.assertFalse(result)

        self.cur2.execute('CREATE SEQUENCE serial START 101;')
        self.conn2.commit()

        result, differences = self.checker.check_sequences()
        self.assertTrue(result)

    def test_check_indexes(self):
        self.cur1.execute(
            'CREATE TABLE schema_foo.bar '
            '(id smallint, value integer, name varchar(100));'
            'CREATE UNIQUE INDEX name_idx ON schema_foo.bar (name);')
        self.conn1.commit()

        result, differences = self.checker.check_indexes()
        self.assertFalse(result)

        self.cur2.execute(
            'CREATE TABLE schema_foo.bar '
            '(id smallint, value integer, name varchar(100));')
        self.conn2.commit()

        result, differences = self.checker.check_indexes()
        self.assertFalse(result)

        self.cur2.execute(
            'CREATE UNIQUE INDEX name_idx ON schema_foo.bar (name);')
        self.conn2.commit()

        result, differences = self.checker.check_indexes()
        self.assertTrue(result)

    def test_check_triggers(self):
        self.cur1.execute('DROP FUNCTION IF EXISTS trigger_function();')
        self.conn1.commit()
        self.cur2.execute('DROP FUNCTION IF EXISTS trigger_function();')
        self.conn2.commit()

        result, differences = self.checker.check_triggers()
        self.assertTrue(result)

        self.cur1.execute(
            """CREATE FUNCTION trigger_function() RETURNS trigger AS
            $$
            BEGIN
            select ("a");
            END;
            $$
            LANGUAGE 'plpgsql';
            CREATE TABLE schema_foo.bar (id smallint,
            value integer, name varchar(100));
            CREATE TRIGGER trig
            BEFORE UPDATE ON schema_foo.bar
            FOR EACH ROW
            EXECUTE PROCEDURE trigger_function();""")
        self.conn1.commit()

        result, differences = self.checker.check_triggers()
        self.assertFalse(result)

        self.cur2.execute(
            """CREATE FUNCTION trigger_function() RETURNS trigger AS
            $$
            BEGIN
            select ("a");
            END;
            $$
            LANGUAGE 'plpgsql';
            CREATE TABLE schema_foo.bar
            (id smallint, value integer, name varchar(100));
            CREATE TRIGGER trig
            BEFORE UPDATE ON schema_foo.bar
            FOR EACH ROW
            EXECUTE PROCEDURE trigger_function();""")
        self.conn2.commit()

        result, differences = self.checker.check_triggers()
        self.assertTrue(result)

    def test_check_functions(self):

        self.cur1.execute('DROP FUNCTION IF EXISTS add(integer, integer);')
        self.conn1.commit()
        self.cur2.execute('DROP FUNCTION IF EXISTS add(integer, integer);')
        self.conn2.commit()

        result, differences = self.checker.check_functions()
        self.assertTrue(result)

        self.cur1.execute(
            """CREATE FUNCTION add(integer, integer) RETURNS integer
            AS 'select $1 + $2;'
            LANGUAGE SQL
            IMMUTABLE
            RETURNS NULL ON NULL INPUT;""")

        self.conn1.commit()

        result, differences = self.checker.check_functions()
        self.assertFalse(result)

        self.cur2.execute(
            """CREATE FUNCTION add(integer, integer) RETURNS integer
            AS 'select $1 + $2;'
            LANGUAGE SQL
            IMMUTABLE
            RETURNS NULL ON NULL INPUT;""")
        self.conn2.commit()

        result, differences = self.checker.check_functions()
        self.assertTrue(result)

    def test_check_rules(self):
        self.cur1.execute('DROP RULE IF EXISTS foorule ON schema_foo.bar;')
        self.conn1.commit()
        self.cur2.execute('DROP RULE IF EXISTS foorule ON schema_foo.bar;')
        self.conn2.commit()

        result, differences = self.checker.check_rules()
        self.assertTrue(result)

        sql = """CREATE TABLE schema_foo.bar
              (id smallint, value integer, name varchar(100));
              CREATE RULE foorule AS ON UPDATE TO
              schema_foo.bar DO ALSO NOTIFY bar;"""

        self.cur1.execute(sql)
        self.conn1.commit()

        result, differences = self.checker.check_rules()
        self.assertFalse(result)

        self.cur2.execute(sql)
        self.conn2.commit()

        result, differences = self.checker.check_rules()
        self.assertTrue(result)

    def test_exclude_schema(self):
        self.test_check_rules()

        result, differences = self.checker.check_rules()
        self.assertTrue(result)

        self.cur1.execute("""CREATE TABLE public.bar
                          (id smallint, value integer, name varchar(100));
                          CREATE RULE foorule AS ON UPDATE TO
                          public.bar DO ALSO NOTIFY bar;""")
        self.conn1.commit()

        result, differences = self.checker.check_rules()
        self.assertFalse(result)

        checker2 = Checker(pg_service1, pg_service2, ['public'])
        result, differences = checker2.check_rules()
        self.assertTrue(result)

        self.cur1.execute("""DROP TABLE IF EXISTS public.bar;""")
        self.conn1.commit()

        result, differences = self.checker.check_rules()
        self.assertTrue(result)

if __name__ == '__main__':
    unittest.main()



import os
import shutil
import unittest

import psycopg2
import psycopg2.extras

from pum.core.dumper import Dumper


class TestDumper(unittest.TestCase):
    """Test the class Dumper.

    2 pg_services needed for test related to empty db:
        pum_test_1
        pum_test_2
    """

    def tearDown(self):
        self.cur1.execute('DROP SCHEMA IF EXISTS test_dumper CASCADE;')
        self.conn1.commit()
        self.conn1.close()

        self.cur2.execute('DROP SCHEMA IF EXISTS test_dumper CASCADE;')
        self.conn2.commit()
        self.conn2.close()

    def setUp(self):
        self.pg_service1 = 'pum_test_1'
        self.pg_service2 = 'pum_test_2'

        self.conn1 = psycopg2.connect("service={0}".format(self.pg_service1))
        self.cur1 = self.conn1.cursor()

        self.conn2 = psycopg2.connect("service={0}".format(self.pg_service2))
        self.cur2 = self.conn2.cursor()

        self.cur1.execute("""
            DROP SCHEMA IF EXISTS test_dumper CASCADE;
            CREATE SCHEMA test_dumper;
            CREATE TABLE test_dumper.dumper_table
                (
                id serial NOT NULL,
                version character varying(50),
                description character varying(200) NOT NULL,
                type integer NOT NULL
                );
            """)
        self.conn1.commit()

        self.cur2.execute("""
            DROP SCHEMA IF EXISTS test_dumper CASCADE;""")
        self.conn2.commit()

        try:
            shutil.rmtree('/tmp/test_dumper')
        except Exception:
            pass

        os.mkdir('/tmp/test_dumper/')

    def test_dump_restore(self):
        dumper = Dumper(self.pg_service1, '/tmp/test_dumper/dump.sql')
        dumper.pg_backup(exclude_schema=['public'])

        dumper = Dumper(self.pg_service2, '/tmp/test_dumper/dump.sql')
        dumper.pg_restore()

        # postgres > 9.4
        self.cur2.execute(
            "SELECT to_regclass('{}');".format('test_dumper.dumper_table'))
        self.assertIsNotNone(self.cur2.fetchone()[0])

if __name__ == '__main__':
    unittest.main()



import os
import shutil
import unittest

import psycopg2
import psycopg2.extras
from pum.core.upgrader import Upgrader, Delta


class TestUpgrader(unittest.TestCase):
    """Test the class Upgrader.

    1 pg_service needed for test:
        pum_test_1
    """

    def tearDown(self):
        del self.upgrader

        self.cur1.execute('DROP SCHEMA IF EXISTS test_upgrader CASCADE;')
        self.conn1.commit()
        self.conn1.close()

    def setUp(self):
        pg_service1 = 'pum_test_1'
        self.upgrades_table = 'test_upgrader.upgrades'

        self.conn1 = psycopg2.connect("service={0}".format(pg_service1))
        self.cur1 = self.conn1.cursor()

        self.cur1.execute("""
            DROP SCHEMA IF EXISTS test_upgrader CASCADE;
            CREATE SCHEMA test_upgrader;
            CREATE TABLE {}
                (
                id serial NOT NULL,
                version character varying(50),
                description character varying(200) NOT NULL,
                type integer NOT NULL,
                script character varying(1000) NOT NULL,
                checksum character varying(32) NOT NULL,
                installed_by character varying(100) NOT NULL,
                installed_on timestamp without time zone NOT NULL DEFAULT now(),
                execution_time integer NOT NULL,
                success boolean NOT NULL,
                CONSTRAINT upgrades_pk PRIMARY KEY (id)
                );
            """.format(self.upgrades_table))
        self.conn1.commit()

        try:
            shutil.rmtree('/tmp/pum_deltas_1')
            shutil.rmtree('/tmp/pum_deltas_2')
        except OSError:
            pass

        os.mkdir('/tmp/pum_deltas_1/')
        os.mkdir('/tmp/pum_deltas_2/')

        with open('/tmp/pum_deltas_1/delta_0.0.1_0.sql', 'w+') as f:
            f.write('DROP TABLE IF EXISTS test_upgrader.bar;')
            f.write(
                'CREATE TABLE test_upgrader.bar '
                '(id smallint, value integer, name varchar(100));')

        with open('/tmp/pum_deltas_1/delta_0.0.1_a.sql', 'w+') as f:
            f.write('SELECT 2;')

        with open('/tmp/pum_deltas_1/delta_0.0.1_1.sql', 'w+') as f:
            f.write('SELECT 1;')

        with open('/tmp/pum_deltas_2/delta_0.0.1_0.sql', 'w+') as f:
            f.write('SELECT 3;')

        self.upgrader = Upgrader(
            pg_service1, self.upgrades_table, ['/tmp/pum_deltas_1/', '/tmp/pum_deltas_2/'])
        self.upgrader.set_baseline('0.0.1')

    def test_upgrader_run(self):
        self.upgrader.run()
        # postgres > 9.4
        self.cur1.execute(
            "SELECT to_regclass('{}');".format(self.upgrades_table))
        self.assertIsNotNone(self.cur1.fetchone()[0])

        self.cur1.execute(
            "SELECT description from {};".format(self.upgrades_table))
        results = self.cur1.fetchall()
        self.assertEqual(len(results), 5)
        self.assertEqual(results[0][0], 'baseline')
        self.assertEqual(results[1][0], '0')
        self.assertEqual(results[2][0], '1')
        self.assertEqual(results[3][0], 'a')
        self.assertEqual(results[4][0], '0')

    def test_delta_valid_name(self):
        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.py'))
        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.sql'))

        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.pre.py'))
        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.pre.sql'))

        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.post.py'))
        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_17072017.post.sql'))

        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0.sql'))
        self.assertTrue(
            Delta.is_valid_delta_name('delta_1.1.0_blahblah_foo_bar.sql'))

        self.assertFalse(Delta.is_valid_delta_name('1.1.0_17072017.sql'))
        self.assertFalse(Delta.is_valid_delta_name('Delta_1.1.0_17072017.sql'))
        self.assertFalse(Delta.is_valid_delta_name('delta_1.1.0_17072017'))
        self.assertFalse(Delta.is_valid_delta_name('delta_1.1.0_17072017.post'))
        self.assertFalse(Delta.is_valid_delta_name('delta_1.1.0_17072017.pre'))
        self.assertFalse(Delta.is_valid_delta_name('delta_1.1_17072017.sql'))

    def test_delta_get_version(self):
        delta = Delta('delta_0.0.0_17072017.sql')
        self.assertEqual(delta.get_version(), '0.0.0')

        delta = Delta('delta_1.2.3_17072017.pre.sql')
        self.assertEqual(delta.get_version(), '1.2.3')

        delta = Delta('delta_100.002.9999_17072017.post.sql')
        self.assertEqual(delta.get_version(), '100.002.9999')

    def test_delta_get_name(self):
        delta = Delta('delta_0.0.0_17072017.sql')
        self.assertEqual(delta.get_name(), '17072017')

        delta = Delta('delta_0.0.0_17072017.py')
        self.assertEqual(delta.get_name(), '17072017')

        delta = Delta('delta_0.0.0_.sql')
        self.assertEqual(delta.get_name(), '')

        delta = Delta('delta_0.0.0_.py')
        self.assertEqual(delta.get_name(), '')

        delta = Delta('delta_0.0.0.sql')
        self.assertEqual(delta.get_name(), '')

        delta = Delta('delta_0.0.0.py')
        self.assertEqual(delta.get_name(), '')

        delta = Delta('delta_0.0.0_foo.pre.sql')
        self.assertEqual(delta.get_name(), 'foo')

        delta = Delta('delta_0.0.0_foo.pre.py')
        self.assertEqual(delta.get_name(), 'foo')

        delta = Delta('delta_0.0.0_foo.post.sql')
        self.assertEqual(delta.get_name(), 'foo')

        delta = Delta('delta_0.0.0_foo.post.py')
        self.assertEqual(delta.get_name(), 'foo')

    def test_delta_get_checksum(self):
        file = open('/tmp/foo.bar', 'w+')
        delta = Delta('/tmp/foo.bar')
        self.assertEqual(
            delta.get_checksum(), 'd41d8cd98f00b204e9800998ecf8427e')
        file.write('The quick brown fox jumps over the lazy dog')
        file.close()
        self.assertEqual(
            delta.get_checksum(), '9e107d9d372bb6826bd81d3542a419d6')

    def test_delta_get_type(self):
        delta = Delta('delta_0.0.0_17072017.sql')
        self.assertEqual(delta.get_type(), Delta.DELTA_SQL)

        delta = Delta('delta_0.0.0_17072017.py')
        self.assertEqual(delta.get_type(), Delta.DELTA_PY)

        delta = Delta('delta_0.0.0_17072017.pre.sql')
        self.assertEqual(delta.get_type(), Delta.DELTA_PRE_SQL)

        delta = Delta('delta_0.0.0_17072017.pre.py')
        self.assertEqual(delta.get_type(), Delta.DELTA_PRE_PY)

        delta = Delta('delta_0.0.0_17072017.post.sql')
        self.assertEqual(delta.get_type(), Delta.DELTA_POST_SQL)

        delta = Delta('delta_0.0.0_17072017.post.py')
        self.assertEqual(delta.get_type(), Delta.DELTA_POST_PY)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
import re
import os
from os import listdir
from os.path import isfile, join, basename, dirname
from collections import OrderedDict
import psycopg2
import psycopg2.extras
from hashlib import md5
import importlib.util
import inspect

from .deltapy import DeltaPy


class Upgrader:
    """This class is used to upgrade an existing database using sql delta files.

    Stores the info about the upgrade in a table on the database."""

    def __init__(self, pg_service, upgrades_table, dirs):
        """Constructor

            Parameters
            ----------
            pg_service: str
                The name of the postgres service (defined in pg_service.conf)
                related to the db
            upgrades_table: str
                The name of the table (int the format schema.name) where the
                informations about the upgrades are stored
            dirs: list(str)
                The paths to directories where delta files are stored
        """
        self.pg_service = pg_service
        self.connection = psycopg2.connect("service={}".format(pg_service))
        self.cursor = self.connection.cursor()
        self.upgrades_table = upgrades_table
        self.dirs = dirs

    def run(self, verbose=False):
        if not self.exists_table_upgrades():
            raise UpgradesTableNotFoundError(self.upgrades_table)

        self.__run_pre_all()

        deltas = self.__get_delta_files()
        deltas = [delta for dirname in deltas for delta in deltas[dirname]]
        if not deltas:
            print('No delta files found')

        db_version = self.current_db_version()

        for d in deltas:
            if verbose:
                print('Found delta {}, version {}, type {}'.format(
                    d.get_name(), d.get_version(), d.get_type()))
                print('     Already applied: ', self.__is_applied(d))
                print('     Version greather or equal than current: ',
                      d.get_version() >= db_version)
            if not self.__is_applied(d) and d.get_version() >= db_version:
                print('     Applying delta {} {}...'.format(
                    d.get_version(), d.get_name()), end=' ')

                if d.get_type() in [Delta.DELTA_PRE_SQL, Delta.DELTA_SQL,
                                    Delta.DELTA_POST_SQL]:
                    self.__run_delta_sql(d)
                    print('OK')
                elif d.get_type() in [Delta.DELTA_PRE_PY, Delta.DELTA_PY,
                                      Delta.DELTA_POST_PY]:
                    self.__run_delta_py(d)
                    print('OK')
                else:
                    print('Delta not applied')
            else:
                if verbose:
                    print('Delta not applied')

        self.__run_post_all()

    def exists_table_upgrades(self):
        """Return if the upgrades table exists

        Returns
        -------
        bool
            True if the table exists
            False if the table don't exists"""

        query = """
            SELECT EXISTS (
            SELECT 1
            FROM   information_schema.tables
            WHERE  table_schema = '{}'
            AND    table_name = '{}'
            );
        """.format(self.upgrades_table[:self.upgrades_table.index('.')],
                   self.upgrades_table[self.upgrades_table.index('.')+1:])

        self.cursor.execute(query)
        return self.cursor.fetchone()[0]

    def __get_dbname(self):
        """Return the db name."""
        return self.connection.get_dsn_parameters()['dbname']

    def __get_dbuser(self):
        """Return the db user"""
        return self.connection.get_dsn_parameters()['user']

    def __get_delta_files(self):
        """Search for delta files and return a dict of Delta objects, keyed by directory names."""
        files = [(d, f) for d in self.dirs for f in listdir(d) if isfile(join(d, f))]

        deltas = OrderedDict()
        for d, f in files:
            file_ = join(d, f)

            if not Delta.is_valid_delta_name(file_):
                continue

            delta = Delta(file_)

            if d not in deltas:
                deltas[d] = []
            deltas[d].append(delta)

        # sort delta objects in each bucket
        for d in deltas:
            deltas[d].sort(key=lambda x: (x.get_version(), x.get_type(), x.get_name()))

        return deltas

    def __run_delta_sql(self, delta):
        """Execute the delta sql file on the database"""

        self.__run_sql_file(delta.get_file())
        self.__update_upgrades_table(delta)

    def __run_delta_py(self, delta):
        """Execute the delta py file"""

        self.__run_py_file(delta.get_file(), delta.get_name())
        self.__update_upgrades_table(delta)

    def __run_pre_all(self):
        """Execute the pre-all.py and pre-all.sql files if they exist"""

        # if the list of delta dirs is [delta1, delta2] the pre scripts of delta2 are
        # executed before the pre scripts of delta1

        for d in reversed(self.dirs):
            pre_all_py_path = os.path.join(d, 'pre-all.py')
            if os.path.isfile(pre_all_py_path):
                print('     Applying pre-all.py...', end=' ')
                self.__run_py_file(pre_all_py_path, 'pre-all')
                print('OK')

            pre_all_sql_path = os.path.join(d, 'pre-all.sql')
            if os.path.isfile(pre_all_sql_path):
                print('     Applying pre-all.sql...', end=' ')
                self.__run_sql_file(pre_all_sql_path)
                print('OK')

    def __run_post_all(self):
        """Execute the post-all.py and post-all.sql files if they exist"""

        # if the list of delta dirs is [delta1, delta2] the post scripts of delta1 are
        # executed before the post scripts of delta2

        for d in self.dirs:
            post_all_py_path = os.path.join(d, 'post-all.py')
            if os.path.isfile(post_all_py_path):
                print('     Applying post-all.py...', end=' ')
                self.__run_py_file(post_all_py_path, 'post-all')
                print('OK')

            post_all_sql_path = os.path.join(d, 'post-all.sql')
            if os.path.isfile(post_all_sql_path):
                print('     Applying post-all.sql...', end=' ')
                self.__run_sql_file(post_all_sql_path)
                print('OK')

    def __run_sql_file(self, filepath):
        """Execute the sql file at the passed path

        Parameters
        ----------
        filepath: str
            the path of the file to execute"""

        with open(filepath, 'r') as delta_file:
            self.cursor.execute(delta_file.read())
            self.connection.commit()

    def __run_py_file(self, filepath, module_name):
        """Execute the python file at the passed path

        Parameters
        ----------
        filepath: str
            the path of the file to execute
        module_name: str
            the name of the python module
            """

        # Import the module
        spec = importlib.util.spec_from_file_location(module_name, filepath)
        delta_py = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(delta_py)

        # Get the python file's directory path
        # Note: we add a separator for backward compatibility, as existing DeltaPy subclasses
        # may assume that delta_dir ends with a separator
        dir_ = dirname(filepath) + os.sep

        # Search for subclasses of DeltaPy
        for name in dir(delta_py):
            obj = getattr(delta_py, name)
            if inspect.isclass(obj) and not obj == DeltaPy and issubclass(
                    obj, DeltaPy):

                delta_py_inst = obj(
                    self.current_db_version(), dir_, self.dirs, self.pg_service,
                    self.upgrades_table)
                delta_py_inst.run()

    def show_info(self):
        """Print info about found delta files and about already made upgrades"""

        deltas = self.__get_delta_files()

        table = [['Version', 'Name', 'Type', 'Status']]

        for dir_ in deltas:
            print('delta files in dir: ', dir_)

            for delta in deltas[dir_]:
                line = [delta.get_version(), delta.get_name()]
                if delta.get_type() == Delta.DELTA_PRE_PY:
                    line.append('pre py')
                elif delta.get_type() == Delta.DELTA_PRE_SQL:
                    line.append('pre sql')
                elif delta.get_type() == Delta.DELTA_PY:
                    line.append('delta py')
                elif delta.get_type() == Delta.DELTA_SQL:
                    line.append('delta sql')
                elif delta.get_type() == Delta.DELTA_POST_PY:
                    line.append('post py')
                elif delta.get_type() == Delta.DELTA_POST_SQL:
                    line.append('post sql')

                if self.__is_applied(delta):
                    line.append('Applied')
                else:
                    line.append('Pending')

                table.append(line)

        self.__print_table(table)

        print('')
        print('Applied upgrades in database')

        query = """SELECT
                version,
                description,
                type,
                installed_by,
                installed_on,
                success
                FROM {}
                """.format(self.upgrades_table)

        self.cursor.execute(query)
        records = self.cursor.fetchall()

        table = [['Version', 'Name', 'Type', 'Installed by', 'Installed on',
                  'Status']]

        for i in records:
            line = [str(i[0]), str(i[1])]
            delta_type = i[2]
            if delta_type == 0:
                line.append('baseline')
            elif delta_type == Delta.DELTA_PRE_PY:
                line.append('pre py')
            elif delta_type == Delta.DELTA_PRE_SQL:
                line.append('pre sql')
            elif delta_type == Delta.DELTA_PY:
                line.append('delta py')
            elif delta_type == Delta.DELTA_SQL:
                line.append('delta sql')
            elif delta_type == Delta.DELTA_POST_PY:
                line.append('post py')
            elif delta_type == Delta.DELTA_POST_SQL:
                line.append('post sql')

            line.append(str(i[3]))
            line.append(str(i[4]))

            success = str(i[5])
            if success == 'True':
                line.append('Success')
            else:
                line.append('Failed')

            table.append(line)

        self.__print_table(table)

    @staticmethod
    def __print_table(table):
        """Print a list in tabular format
        Based on https://stackoverflow.com/a/8356620"""

        col_width = [max(len(x) for x in col) for col in zip(*table)]
        print("| " + " | ".join("{:{}}".format(x, col_width[i])
                                for i, x in enumerate(table[0])) + " |")
        print("| " + " | ".join("{:{}}".format('-' * col_width[i], col_width[i])
                                for i, x in enumerate(table[0])) + " |")
        for line in table[1:]:
            print("| " + " | ".join("{:{}}".format(x, col_width[i])
                                    for i, x in enumerate(line)) + " |")

    def __is_applied(self, delta):
        """Verifies if delta file is already applied on database

        Parameters
        ----------
        delta: Delta object
            The delta object representing the delta file

        Returns
        -------
        bool
            True if the delta is already applied on the db
            False otherwise
        """

        query = """
        SELECT id FROM {}
        WHERE version = '{}'
            AND checksum = '{}'
            AND success = 'TRUE'
        """.format(
            self.upgrades_table, delta.get_version(), delta.get_checksum())

        self.cursor.execute(query)
        if not self.cursor.fetchone():
            return False
        else:
            return True

    def __update_upgrades_table(self, delta):
        """Add a new record into the upgrades information table about the
        applied delta

        Parameters
        ----------
        delta: Delta
            the applied delta file"""

        query = """
        INSERT INTO {} (
            --id,
            version,
            description,
            type,
            script,
            checksum,
            installed_by,
            --installed_on,
            execution_time,
            success
        ) VALUES(
            '{}',
            '{}',
            {},
            '{}',
            '{}',
            '{}',
            1,
            TRUE
        ) """.format(
            self.upgrades_table, delta.get_version(), delta.get_name(),
            delta.get_type(), delta.get_file(), delta.get_checksum(),
            self.__get_dbuser())

        self.cursor.execute(query)
        self.connection.commit()

    def create_upgrades_table(self):
        """Create the upgrades information table"""

        query = """CREATE TABLE IF NOT EXISTS {}
                (
                id serial NOT NULL,
                version character varying(50),
                description character varying(200) NOT NULL,
                type integer NOT NULL,
                script character varying(1000) NOT NULL,
                checksum character varying(32) NOT NULL,
                installed_by character varying(100) NOT NULL,
                installed_on timestamp without time zone NOT NULL DEFAULT now(),
                execution_time integer NOT NULL,
                success boolean NOT NULL,
                PRIMARY KEY (id)
                )
        """.format(self.upgrades_table)

        self.cursor.execute(query)
        self.connection.commit()

    def set_baseline(self, version):
        """Set the baseline into the creation information table

        version: str
            The version of the current database to set in the information
            table. The baseline must be in the format x.x.x where x are numbers.
        """
        pattern = re.compile(r"^\d+\.\d+\.\d+$")
        if not re.match(pattern, version):
            raise ValueError('Wrong version format')

        query = """
                INSERT INTO {} (
                    version,
                    description,
                    type,
                    script,
                    checksum,
                    installed_by,
                    execution_time,
                    success
                ) VALUES(
                    '{}',
                    '{}',
                    {},
                    '{}',
                    '{}',
                    '{}',
                    1,
                    TRUE
                ) """.format(self.upgrades_table, version, 'baseline', 0,
                             '', '', self.__get_dbuser())
        self.cursor.execute(query)
        self.connection.commit()

    def current_db_version(self):
        """Read the upgrades information table and return the current db
        version

        Returns
        -------
        str
            the current db version
        """

        query = """
        SELECT version from {} WHERE success = TRUE ORDER BY version DESC
        """.format(self.upgrades_table)

        self.cursor.execute(query)

        return self.cursor.fetchone()[0]


class Delta:
    """This class represent a delta file."""

    DELTA_PRE_PY = 0
    DELTA_PRE_SQL = 1
    DELTA_PY = 2
    DELTA_SQL = 3
    DELTA_POST_PY = 4
    DELTA_POST_SQL = 5

    FILENAME_PATTERN = (
        r"^(delta_)(\d+\.\d+\.\d+)(_*)(\w*)\."
        r"(pre\.sql|sql|post\.sql|pre\.py|py|post\.py)$")

    @staticmethod
    def is_valid_delta_name(file):
        """Return if a file has a valid name

        A delta file name can be:
        - pre-all.py
        - pre-all.sql
        - delta_x.x.x_ddmmyyyy.pre.py
        - delta_x.x.x_ddmmyyyy.pre.sql
        - delta_x.x.x_ddmmyyyy.py
        - delta_x.x.x_ddmmyyyy.sql
        - delta_x.x.x_ddmmyyyy.post.py
        - delta_x.x.x_ddmmyyyy.post.sql
        - post-all.py
        - post-all.sql

        where x.x.x is the version number and _ddmmyyyy is an optional
        description, usually representing the date of the delta file
        """
        filename = basename(file)
        pattern = re.compile(Delta.FILENAME_PATTERN)
        if re.match(pattern, filename):
            return True
        return False

    def __init__(self, file):
        self.file = file
        filename = basename(self.file)
        pattern = re.compile(self.FILENAME_PATTERN)
        self.match = re.match(pattern, filename)

    def get_version(self):
        """Return the version of the delta file."""
        return self.match.group(2)

    def get_name(self):
        """Return the name (description) of the delta file."""
        return self.match.group(4)

    def get_checksum(self):
        """Return the md5 checksum of the delta file."""
        with open(self.file, 'rb') as f:
            cs = md5(f.read()).hexdigest()
        return cs

    def get_type(self):
        """Return the type of the delta file.

        Returns
        -------
        type: int
        """

        ext = self.match.group(5)

        if ext == 'pre.py':
            return Delta.DELTA_PRE_PY
        elif ext == 'pre.sql':
            return Delta.DELTA_PRE_SQL
        elif ext == 'py':
            return Delta.DELTA_PY
        elif ext == 'sql':
            return Delta.DELTA_SQL
        elif ext == 'post.py':
            return Delta.DELTA_POST_PY
        elif ext == 'post.sql':
            return Delta.DELTA_POST_SQL

    def get_file(self):
        return self.file


class UpgradesTableNotFoundError(Exception):
    """raise this when Upgrades table is not present"""




# From http://code.activestate.com/recipes/541096-prompt-the-user-for
# -confirmation/
def ask_for_confirmation(prompt=None, resp=False):
    """Prompt for a yes or no response from the user.

        Parameters
        ----------
        prompt: basestring
            The question to be prompted to the user.
        resp: bool
            The default value assumed by the caller when user simply
            types ENTER.

        Returns
        -------
        bool
            True if the user response is 'y' or 'Y'
            False if the user response is 'n' or 'N'
    """
    global input
    if prompt is None:
        prompt = 'Confirm'

    if resp:
        prompt = '%s [%s]|%s: ' % (prompt, 'y', 'n')
    else:
        prompt = '%s [%s]|%s: ' % (prompt, 'n', 'y')

    while True:
        # Fix for Python2. In python3 raw_input() is now input()
        try:
            input = raw_input
        except NameError:
            pass
        ans = input(prompt)
        if not ans:
            return resp
        if ans not in ['y', 'Y', 'n', 'N']:
            print('please enter y or n.')
            continue
        if ans == 'y' or ans == 'Y':
            return True
        if ans == 'n' or ans == 'N':
            return False

class Bcolors:
    WAITING = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'


# -*- coding: utf-8 -*-

from __future__ import print_function

import subprocess
from distutils.version import LooseVersion


class Dumper:
    """This class is used to dump and restore a Postgres database."""

    def __init__(self, pg_service, file):
        self.file = file

        self.pg_service = pg_service

    def pg_backup(self, pg_dump_exe='pg_dump', exclude_schema=None):
        """Call the pg_dump command to create a db backup

        Parameters
        ----------
        pg_dump_exe: str
            the pg_dump command path
        exclude_schema: str[]
            list of schemas to be skipped
        """

        command = [
            pg_dump_exe, '-Fc', '-f', self.file,
            'service={}'.format(self.pg_service)
            ]
        if exclude_schema:
            command.append(' '.join("--exclude-schema={}".format(schema) for schema in exclude_schema))

        subprocess.check_output(command, stderr=subprocess.STDOUT)

    def pg_restore(self, pg_restore_exe='pg_restore', exclude_schema=None):
        """Call the pg_restore command to restore a db backup

        Parameters
        ----------
        pg_restore_exe: str
            the pg_restore command path
        """

        command = [
            pg_restore_exe, '-d',
            'service={}'.format(self.pg_service),
            '--no-owner'
            ]

        if exclude_schema:
            exclude_schema_available = False
            try:
                pg_version = subprocess.check_output(['pg_restore','--version'])
                pg_version = str(pg_version).replace('\\n', '').replace("'", '').split(' ')[-1]
                exclude_schema_available = LooseVersion(pg_version) >= LooseVersion("10.0")
            except subprocess.CalledProcessError as e:
                print("*** Could not get pg_restore version:\n", e.stderr)
            if exclude_schema_available:
                command.append(' '.join("--exclude-schema={}".format(schema) for schema in exclude_schema))
        command.append(self.file)

        try:
            subprocess.check_output(command)
        except subprocess.CalledProcessError as e:
            print("*** pg_restore failed:\n", command, '\n', e.stderr)



from setuptools import setup
import sys
if sys.version_info < (3,3):
    sys.exit('Pum requires at least Python version 3.3.\nYou are currently running this installation with\n\n{}'.format(sys.version))

setup(
    name = 'pum',
    packages = [
        'pum',
        'pum/core',
        'pum/utils'
    ],
    scripts = [
        'scripts/pum'
    ],
    version = '0.6.6',
    description = 'Postgres upgrade manager',
    author = 'Mario Baranzini',
    author_email = 'mario@opengis.ch',
    url = 'https://github.com/opengisch/pum',
    download_url = 'https://github.com/opengisch/pum/archive/0.6.6.tar.gz', # I'll explain this in a second
    keywords = [
        'postgres',
        'migration',
        'upgrade'
    ],
    classifiers = [
        'Topic :: Database',
        'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
        'Intended Audience :: System Administrators',
        'Intended Audience :: Information Technology',
        'Topic :: Software Development :: Version Control',
        'Development Status :: 5 - Production/Stable'
    ],
    install_requires = [
        'psycopg2-binary>=2.7.3',
        'PyYAML'
    ],
    python_requires=">=3.3",
)
